<?php

namespace QUI\Demodata\Parser;

use QUI\Demodata\Exceptions\UnknownFileFormatException;
use QUI\Demodata\Parser\XML\BricksParser;
use QUI\Demodata\Parser\XML\EventParser;
use QUI\Demodata\Parser\XML\ProjectParser;
use QUI\Package\Package;
use QUI\Projects\Project;

use function pathinfo;

/**
 * Class DemoDataParser
 * @package QUI\Demodata\Parser
 */
class DemoDataParser
{
    /**
     * Attempts to parse the given file. It will select the appropriate parser by file extension.
     * **Return format:**
     * ```
     * [
     *  'bricks' => array,
     *  'projects' => array,
     * ]
     * ```
     *
     * @param Package $TemplatePackage
     * @param Project $Project
     *
     * @return array
     * @throws UnknownFileFormatException
     */
    public function parse(Package $TemplatePackage, Project $Project): array
    {
        $demoDataFilePath = $TemplatePackage->getDir() . 'demodata.xml';
        $fileExtension = pathinfo($demoDataFilePath, PATHINFO_EXTENSION);

        switch ($fileExtension) {
            case 'xml':
                return $this->parseXML($TemplatePackage, $Project);

            default:
                throw new UnknownFileFormatException([
                    'quiqqer/demodata',
                    'exception.file.format.extension.unknown'
                ]);
        }
    }

    /**
     * Attempts to parse the given XML file.
     * **Return format:**
     * ```
     * [
     *  'bricks' => array,
     *  'projects' => array,
     * ]
     * ```
     *
     * @param Package $TemplatePackage
     * @param Project $Project
     * @return array
     */
    protected function parseXML(Package $TemplatePackage, Project $Project): array
    {
        $filePath = $TemplatePackage->getDir() . 'demodata.xml';
        $langFilePath = $TemplatePackage->getDir() . 'demodata_' . $Project->getLang() . '.xml';
        $data = [];

        if (file_exists($langFilePath)) {
            $filePath = $langFilePath;
        }

        $data['meta'] = [
            'file' => $filePath,
            'template' => [
                'name' => $TemplatePackage->getName(),
                'path' => $TemplatePackage->getDir()
            ]
        ];

        $data['projects'] = ProjectParser::parseProjects($filePath);
        $data['bricks'] = BricksParser::parseBricks($filePath);
        $data['events'] = EventParser::parseEvents($filePath);

        return $data;
    }
}
